//----------------------------------------------
// NFC Packet System with TRT Implementation
//----------------------------------------------

#include "NFCPack.h"

ISR (TIMER0_COMPA_vect)
{
	unsigned char c ;
	
	//**********************
  	// send an nfc char nfc tx is ready and still char in buffer to send
	// and USART is ready
	if (nfc_tx_ready ){
		if (UCSR1A & (1<<UDRE1)) UDR1 = nfc_tx_buffer[nfc_tx_count++];
		if (nfc_tx_buffer[nfc_tx_count]==0x00) nfc_tx_ready = 0 ; // end of buffer
		if (nfc_tx_count >= buffer_size) nfc_tx_ready = 0; // buffer overrun
	}
	
	//**********************
  	// recv an nfc char if data ready 
  	// otherwise set c to null 
	if (UCSR1A & (1<<RXC1) ) {
		c = UDR1 ; // valid char 
	}
	else c = 0 ; // nonvalid

	//**********************
	// append character to the received string
	// if character is valid and we expect a string
	if (c>0) {

		if (c == start_token) { // restart the string
			nfc_rx_count = 0 ;
		}

		else if (c == end_token){ // end the string
			nfc_rx_buffer[nfc_rx_count] = 0x00 ;
			nfc_rx_ready = 1 ;
		}

		else { // add to string and check for buffer overrun
			nfc_rx_buffer[nfc_rx_count++] = c ;
			if (nfc_rx_count >= buffer_size) { // buffer overrun
				nfc_rx_ready = 2;
				nfc_rx_buffer[buffer_size-1] = 0x00 ;
				nfc_rx_count = buffer_size -1 ;
			}
		}

	} // end if c>0	
}

// Transmit Packet
// Input transmitter id and string packet payload
void nfc_send_packet(char tx_id, char data[])
{
	char tx_ck_sum, tx_ck_sum1, tx_ck_sum2 ;
	char i ;
	
	// compute check sum on input data string only
	tx_ck_sum = 0;
	for (i=0; i<strlen(data); i++)
		tx_ck_sum ^= data[i] ;
	// chop the check sum into two 1/2 bytes and add a bit
	tx_ck_sum1 = 0x10 + (tx_ck_sum & 0x0f) ;
	tx_ck_sum2 = 0x10 + ((tx_ck_sum>>4) & 0x0f) ;

	// format send string
  	sprintf(nfc_tx_buffer,"%c%c%s%c%c%c", start_token, tx_id, data, tx_ck_sum1, tx_ck_sum2, end_token );
	nfc_tx_count = 0 ;
  	nfc_tx_ready = 1 ;
	
	tx_send_time = trtCurrentTime();
	// wait 
	while (nfc_tx_ready && (trtCurrentTime() < tx_send_time + nfc_tx_timeout)) {};	
}

// Receive Packet
// input expected transmitter id
// returns zero if payload is valid and returns payload
// 1 means no data; 2 means buffer overrun; 3 means bad tx id; 4 means bad checksum
char nfc_rec_packet(char tx_id, char data[])
{
	char rx_status = 0;
	char rx_ck_sum, rx_ck_sum_ref;
	char i ;
	char buf_len ;

	buf_len = (char)strlen(nfc_rx_buffer) ;
	if (buf_len >= buffer_size) buf_len = buffer_size-1;

	// error check
	if (nfc_rx_ready == 2) {rx_status = 2; return rx_status;} // buffer overrun
	if (nfc_rx_ready == 0) {rx_status = 1; return rx_status;} // no data -- timeout
	if (nfc_rx_buffer[0] != tx_id) {rx_status = 3; return rx_status;} // bad transmitter id
	if (nfc_rx_ready != 1) {rx_status = 1; return rx_status;} // invalid data

	// compute receive checksum
	rx_ck_sum = 0 ;
	for (i=1; i<buf_len-2; i++)
		rx_ck_sum ^= nfc_rx_buffer[i] ;
	
	rx_ck_sum_ref = (nfc_rx_buffer[buf_len-2] & 0x0f) +
		 (nfc_rx_buffer[buf_len-1]<<4) ;

    if (rx_ck_sum_ref != rx_ck_sum) 
		{rx_status = 4; return rx_status;}  // bad check sum

	// set up the valid data return stuff
	nfc_rx_buffer[0] = ' '; // strip the transmit id
	nfc_rx_buffer[buf_len-2] = 0x00; // strip the check sum and trailer
	nfc_rx_ready = 0 ;
	// trim initial space using ir_rx_buffer+1
	strlcpy(data, nfc_rx_buffer+1, buffer_size) ; 

	return rx_status ;
}

// NFC Packet System Initialization
void nfc_init(void) 
{
	//set up timer 0 for 16 mSec timebase 
    TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
    OCR0A = 249 ;//
    //set prescalar to divide by 64 (4 microsec/tick)
    TCCR0B= 5;	
    // turn on clear-on-match
    TCCR0A= (1<<WGM01) ;
	
	// init UART1 for NFC comm
	unsigned long baud_num = (F_CPU / (16UL * NFC_UART_BAUD)) - 1;
    UBRR1L = baud_num & 0xff;
	UBRR1H = baud_num >> 8; 
    UCSR1B = _BV(TXEN1) | _BV(RXEN1); /* tx/rx enable */
    UCSR1C = (1<<UCSZ11) | (1<<USBS1) ; // 7 bit | 2 stop bits
}
