//----------------------------------------------
// NFC Module Test Program
// Written by Lingnan Liu
// Version 0.1 at 12/01/2013
//----------------------------------------------

// The TRT library files, do not add them to project
#include "trtSettings.h"
#include "trtkernel_1284.c"

// Ad-hoc NFC Module Library and PC Comm Library
#include "NFCPack.h"
#include "PCComm.h"

// UART file descriptor
FILE uart0 = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// Semaphores for transmitting and receiving
#define SEM_SEND 3
#define SEM_RECV 4
// Semaphores to protect shared variables
#define SEM_SHARED 5

// Input arguments
int args[3];

// Transmitter Buffer
char send_buffer[64];
char send_tx_data[64];
char send_rx_data[64];

// Receiver Buffer
char recv_buffer[64];
char recv_tx_data[64];
char recv_rx_data[64]; // temporal data

// States
#define IDLE 100
#define SEND 101
#define RECV 102
#define DONE 103
unsigned transmitter_state = IDLE, receiver_state = RECV;

// Flags
char data_tx_ready = 0;

//-----------------------------------------------------------
// Serial Comm with PC - accept PC input and store the params
//-----------------------------------------------------------
void serialComm(void* args) {
	// variables to store cmds and setting params
	uint32_t rel, dead;
	char param[64];
	char cmd[4];
  
	// Detect the cmds
	while(1) {
		fprintf(&uart0, ">");
		fscanf(&uart0, "%s %s", cmd, param);
		
		trtWait(SEM_SHARED);
		if (cmd[0] == 's')
		{
			sprintf(send_buffer, "%s", param);
			fprintf(&uart0, "Sending: %s\n", send_buffer);
			// begin transmitting
			data_tx_ready = 1;
		}
		trtSignal(SEM_SHARED);
		
		// Sleep
		rel = trtCurrentTime() + SECONDS2TICKS(0.1);
		dead = trtCurrentTime() + SECONDS2TICKS(0.2);
		trtSleepUntil(rel, dead);
  }
}

void transmitter(void* arg)
{
	uint32_t rel, dead;

	// Send the text in send_buffer
	while(1)
	{
		switch(transmitter_state)
		{
			case IDLE:
				if (data_tx_ready) transmitter_state = SEND;
				break;
			case SEND:
				// read from the send_buffer
				trtWait(SEM_SHARED);
				strcpy(send_tx_data, send_buffer);
				trtSignal(SEM_SHARED);
				// transmit
				trtWait(SEM_SEND);
				nfc_send_packet('T', send_tx_data);
				trtSignal(SEM_SEND);
				// back to IDLE state
				trtWait(SEM_SHARED);
				data_tx_ready = 0;
				trtSignal(SEM_SHARED);
				transmitter_state = IDLE;
				break;
			default:
				transmitter_state = IDLE;
				break;
		}

		// Sleep
	    rel = trtCurrentTime() + SECONDS2TICKS(0.1);
	    dead = trtCurrentTime() + SECONDS2TICKS(0.2);
	    trtSleepUntil(rel, dead);
	}
}

void receiver(void* arg)
{
	uint32_t rel, dead;
	char recv_rx_status;
	
    while (1)
	{
		switch(receiver_state)
		{
			case RECV:
				// try to receive data
				trtWait(SEM_RECV);
				recv_rx_status = nfc_rec_packet('T', recv_rx_data);
				trtSignal(SEM_RECV);
				// if successfully received
				if (recv_rx_status == 0)
				{
					// to DONE state
					strcpy(recv_buffer, recv_rx_data);
					fprintf(&uart0, "Receive Completed\n");
					fprintf(&uart0, "Received: %s\n", recv_buffer);
					receiver_state = DONE;
				}
				break;
			case DONE:
				// back to RECV state
				receiver_state = RECV;
				break;
			default:
				receiver_state = RECV;
				break;
		}

		// Sleep
	    rel = trtCurrentTime() + SECONDS2TICKS(0.1);
	    dead = trtCurrentTime() + SECONDS2TICKS(0.2);
	    trtSleepUntil(rel, dead);
	}
}

// Initialization and Idle Task
int main(void) {
	
	// Initialize the NFC Module and usart
	nfc_init();
	trt_uart_init();
	fprintf(&uart0,"\n\r NFC Module Test\n\r\n\r");
	
	// start TRT
    trtInitKernel(300); // 300 bytes for the idle task stack
	
	// Create Tasks and Semaphores
	trtCreateTask(serialComm, 300, SECONDS2TICKS(0.1), SECONDS2TICKS(0.2), &(args[0]));
	trtCreateTask(transmitter, 300, SECONDS2TICKS(0.1), SECONDS2TICKS(0.2), &(args[0]));
	trtCreateTask(receiver,300, SECONDS2TICKS(0.1), SECONDS2TICKS(0.2), &(args[2]));
	
	// Must creat the first two semaphores if you use the uart
	trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
	trtCreateSemaphore(SEM_STRING_DONE,0);  // user typed <enter>
	
	trtCreateSemaphore(SEM_SEND,1);
	trtCreateSemaphore(SEM_RECV,1);
	// Create the semaphore to protect shared variables
	trtCreateSemaphore(SEM_SHARED, 1);
	
	// --- Idle task --------------------------------------
	uint32_t rel, dead;
    while (1) {

		// Sleep
	    rel = trtCurrentTime() + SECONDS2TICKS(0.1);
	    dead = trtCurrentTime() + SECONDS2TICKS(0.2);
	    trtSleepUntil(rel, dead);
	}
}
